local ADDON = ...
local tooltip

local string_format = string.format

local OnlineIcon = "|TInterface\\FriendsFrame\\StatusIcon-Online:12:12:0:0:16:16:0:16:0:16|t "
local AwayIcon = "|TInterface\\FriendsFrame\\StatusIcon-Away:12:12:0:0:16:16:0:16:0:16|t "
local DnDIcon = "|TInterface\\FriendsFrame\\StatusIcon-DnD:12:12:0:0:16:16:0:16:0:16|t "
local StatusIcon

local arg = {}

local LibQTip = LibStub('LibQTip-1.0')
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")

local dataobj = ldb:NewDataObject(ADDON, {
	type = "data source",
	icon = "Interface\\Addons\\"..ADDON.."\\icon.tga",
	text = "-"
})

local function classcolor(name,class)
	if class then 
		class = strupper(class):gsub("%s+","")
		local color = _G["RAID_CLASS_COLORS"][class]
		return string_format("\124cff%02x%02x%02x%s\124r", color.r*255, color.g*255, color.b*255, name)
	else 
		return name
	end
end

local function Button_OnClick(row,arg,button)
	
	if arg.type == "GUILD" or arg.type == "FRIENDS" then 	
		if button == "LeftButton" and IsAltKeyDown() then
			print("You are inviting " .. classcolor(arg.name,arg.class) .. " in your group.")
			C_PartyInfo.InviteUnit(arg.name)
		else
			ChatFrame_SendTell(arg.name)
		end
	end
	
	if arg.type == "BNET" then 
		if arg.client == "WoW" then 
			if button == "LeftButton" and IsAltKeyDown() then
				print("You are inviting " .. arg.name .. " in your group.")
				C_PartyInfo.InviteUnit(arg.name)
			else 
				ChatFrame_SendBNetTell(arg.account)
			end
		else
			ChatFrame_SendBNetTell(arg.account)
		end
	end	
	
end

local function OnRelease(self)
	LibQTip:Release(self.tooltip)
	self.tooltip = nil  
end  
 
local function OnLeave(self)
	if self.tooltip and not self.tooltip:IsMouseOver() then
		self.tooltip:Release()
	end
end  


local function anchor_OnEnter(self)
	
	arg = {}

	if self.tooltip then
		LibQTip:Release(self.tooltip)
		self.tooltip = nil  
	end

    local row,col
    local tooltip = LibQTip:Acquire(ADDON.."tip", 4, "LEFT", "LEFT", "LEFT","LEFT")
    self.tooltip = tooltip 
    tooltip:SmartAnchorTo(self)
	tooltip:EnableMouse(true)
	tooltip.OnRelease = OnRelease
	tooltip.OnLeave = OnLeave
    tooltip:SetAutoHideDelay(1, self)

-- ### Guild #######################################################################################

	if IsInGuild() then 
	
		row,col = tooltip:AddHeader()
		tooltip:SetCell(row,1,select(1,GetGuildInfo("player")),"LEFT",4)
		tooltip:SetCellTextColor(row,1,0,1,0)
		
		if GetGuildRosterMOTD() then 
			row,col = tooltip:AddLine()
			tooltip:SetCell(row,1,GetGuildRosterMOTD(),"LEFT",4)
			tooltip:SetCellTextColor(row,1,0,1,0)
		end

		row,col = tooltip:AddLine("")
		row,col = tooltip:AddLine("name","level","zone","rank")
		tooltip:SetLineTextColor(row, 1, 0, 0)

		row,col = tooltip:AddLine("")
		
		for i = 1, GetNumGuildMembers() do
			local name, rank, rankIndex, level, class, zone, note, officernote, online, status, classFileName, achievementPoints, achievementRank, isMobile = GetGuildRosterInfo(i)

			if online then
				name = Ambiguate(name, "none")
				
				if status == 1 then
					StatusIcon = AwayIcon
				elseif status == 2 then 
					StatusIcon = DnDIcon
				else
					StatusIcon = ""
				end			
				
				row,col = tooltip:AddLine(StatusIcon .. classcolor(name,classFileName),level,zone,rank)
		
				arg[row] = { type = "GUILD", name = name, class = classFileName, client = "-", account = "-"}
				tooltip:SetLineScript(row, 'OnMouseUp', Button_OnClick, arg[row])

			end
		end	
		
		row,col = tooltip:AddLine("")		
		row,col = tooltip:AddLine("")
		row,col = tooltip:AddSeparator()
		row,col = tooltip:AddLine("")
		row,col = tooltip:AddLine("")	
	end
	-- ### BNET Friends #######################################################################################

	local numBNetTotal, numBNetOnline, numBNetFavorite, numBNetFavoriteOnline = BNGetNumFriends()
	local bnet_color = {r =".2",g= ".6",b="1"}

	if numBNetTotal > 0 then 

		row,col = tooltip:AddLine("")
		
		row,col = tooltip:AddHeader()
		tooltip:SetCell(row,1,"BNET Friends online","LEFT",4)
		tooltip:SetCellTextColor(row,1,bnet_color.r,bnet_color.g,bnet_color.b)	

		row,col = tooltip:AddLine("")		
		row,col = tooltip:AddLine("")

		row,col = tooltip:AddLine("accountName","battleTag","zone","client")
		tooltip:SetLineTextColor(row, 1, 0, 0)

		row,col = tooltip:AddLine("")
		
		local numBNetTotal, numBNetOnline, numBNetFavorite, numBNetFavoriteOnline = BNGetNumFriends()

		for i = 1, numBNetTotal do

			-- https://wow.gamepedia.com/API_C_BattleNet.GetFriendAccountInfo
			local accountInfo = C_BattleNet.GetFriendAccountInfo(i)

			local bnetAccID = accountInfo.gameAccountID
			local account = accountInfo.accountName
			local isOnline = accountInfo.gameAccountInfo.isOnline
			local client = accountInfo.gameAccountInfo.clientProgram
			local character = accountInfo.gameAccountInfo.characterName
			local btag = accountInfo.battleTag
			local timestamp = date("%d.%m.%y %H:%M:%S", accountInfo.lastOnlineTime)	
			local class = accountInfo.gameAccountInfo.className
			local richPresence = accountInfo.gameAccountInfo.richPresence
			
			if isOnline then
							
				if character == nil then 
					character = account 
				end

				row,col = tooltip:AddLine(BNet_GetClientEmbeddedTexture(client, 14) .. classcolor(character,class),btag,richPresence,client)
				
				if client == "WoW" then
					local realmName = "-" .. accountInfo.gameAccountInfo.realmName
					arg[row] = { type = "BNET", account = account, name = character .. realmName, class = class or "-", client = client }
				else 
					arg[row] = { type = "BNET", account = account, name = "-", class = "-", client = client }
				end
				tooltip:SetLineScript(row, 'OnMouseUp', Button_OnClick, arg[row])
				
				-- BATTLENET_FONT_COLOR_CODE 
				if client == "App" or client == "BSAp" then 
					tooltip:SetLineTextColor(row,bnet_color.r,bnet_color.g,bnet_color.b)
				else
					tooltip:SetLineTextColor(row, 1, 1, 1)
				end	
			end
		end	

		row,col = tooltip:AddLine("")
		row,col = tooltip:AddLine("")		
		row,col = tooltip:AddLine("")
		row,col = tooltip:AddSeparator()
		row,col = tooltip:AddLine("")
	end
	
	-- ### WoW Friends #######################################################################################

	local numTotFriends = C_FriendList.GetNumFriends()
	local numTotOnlineFriends = C_FriendList.GetNumOnlineFriends()

	if numTotOnlineFriends > 0 then 
		
		row,col = tooltip:AddLine("")
		
		row,col = tooltip:AddHeader()
		tooltip:SetCell(row,1,"WoW Friends online","LEFT",4)
		tooltip:SetCellTextColor(row,1,1,1,0)	

		row,col = tooltip:AddLine("")		
		row,col = tooltip:AddLine("")

		row,col = tooltip:AddLine("name","level","zone","class")
		tooltip:SetLineTextColor(row, 1, 0, 0)

		row,col = tooltip:AddLine("")	

		for i = 1, numTotFriends do

			local accountInfo = C_FriendList.GetFriendInfoByIndex(i)
			local area = accountInfo.area
			local account = accountInfo.accountName
			local isOnline = accountInfo.connected
			local level = accountInfo.level
			local name = accountInfo.name
			local class = accountInfo.className
			local isafk = accountInfo.afk
			local isdnd = accountInfo.dnd

			if isOnline then

				if isafk then
					StatusIcon = AwayIcon
				elseif isdnd then 
					StatusIcon = DnDIcon
				else
					StatusIcon = ""
				end
				row,col = tooltip:AddLine(StatusIcon .. classcolor(name,class),level,area,class)

				arg[row] = { type = "FRIENDS", name = name, class = class, client = "-", account = "-"}
				tooltip:SetLineScript(row, 'OnMouseUp', Button_OnClick, arg[row])
				
			end
		end	

	row,col = tooltip:AddLine("")
	row,col = tooltip:AddLine("")		
	row,col = tooltip:AddLine("")
	row,col = tooltip:AddSeparator()
	row,col = tooltip:AddLine("")

	end
	
	-- ### Let's Show #######################################################################################
	
	row,col = tooltip:AddLine()
	tooltip:SetCell(row,1,"L-Click Open |cFF00FF00Guild|r","LEFT",2)
	tooltip:SetCell(row,3,"Name L-Click |cFFFF7FFFWhisp|r","RIGHT",2)	

	row,col = tooltip:AddLine()
	tooltip:SetCell(row,1,"R-Click Open |cFFFFFF00Friends|r","LEFT",2)	
	tooltip:SetCell(row,3,"Name ALT+L-Click |cFFAAAAFFInvite|r","RIGHT",2)
	-- tooltip:SetScript("OnLeave", function(self) self:Release() tooltip = nil end)
	row,col = tooltip:Show()

end
 
dataobj.OnClick = function(self, button)  
	if button == "LeftButton" then
		if not CommunitiesFrame or not CommunitiesFrame:IsShown() then 
			ToggleGuildFrame() 
		end 
		CommunitiesFrame:SetDisplayMode(COMMUNITIES_FRAME_DISPLAY_MODES.ROSTER)
	end	
	
	if button == "RightButton" then
		ToggleFriendsFrame(1)
	end	
end

dataobj.OnEnter = function(self)
	if self.tooltip then
		LibQTip:Release(self.tooltip)
		self.tooltip = nil
	end
	anchor_OnEnter(self)
end

dataobj.OnLeave = function(self)
	-- Null operation: Some LDB displays get cranky if this method is missing.
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:RegisterEvent("PLAYER_GUILD_UPDATE")
frame:RegisterEvent("GUILD_ROSTER_UPDATE")
frame:RegisterEvent("GUILD_RANKS_UPDATE")
frame:RegisterEvent("GUILD_MOTD")
frame:RegisterEvent("FRIENDLIST_UPDATE")
frame:RegisterEvent("BN_FRIEND_ACCOUNT_ONLINE")
frame:RegisterEvent("BN_FRIEND_ACCOUNT_OFFLINE")
frame:SetScript("OnEvent", function(self, event, ...)
		dataobj.text = string_format(" |cFF00FF00%s|r |cFF379BFF%s|r |cFFFFFF00%s|r", "G:"..select(3,GetNumGuildMembers()),"B:"..select(2,BNGetNumFriends()), "F:" .. C_FriendList.GetNumOnlineFriends())
end)